@@ -1,7 +1,7 @@
 /*
  * Copyright (C) 2000-2003 Damien Miller.  All rights reserved.
  * Copyright (C) 1999 WIDE Project.  All rights reserved.
- * 
+ *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
@@ -13,7 +13,7 @@
  * 3. Neither the name of the project nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
- * 
+ *
  * THIS SOFTWARE IS PROVIDED BY THE PROJECT AND CONTRIBUTORS ``AS IS'' AND
  * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
@@ -43,36 +43,37 @@ TCL_DECLARE_MUTEX(netdbMutex)
 static size_t
 strlcpy(char *dst, const char *src, size_t siz)
 {
-        char *d = dst;
-        const char *s = src;
-        size_t n = siz;
+	char *d = dst;
+	const char *s = src;
+	size_t n = siz;
 
-        /* Copy as many bytes as will fit */
-        if (n != 0 && --n != 0) {
-                do {
-                        if ((*d++ = *s++) == 0)
-                                break;
-                } while (--n != 0);
-        }
+	/* Copy as many bytes as will fit */
+	if (n != 0 && --n != 0) {
+		do {
+			if ((*d++ = *s++) == 0)
+				break;
+		} while (--n != 0);
+	}
 
-        /* Not enough room in dst, add NUL and traverse rest of src */
-        if (n == 0) {
-                if (siz != 0)
-                        *d = '\0';              /* NUL-terminate dst */
-                while (*s++)
-                        ;
-        }
+	/* Not enough room in dst, add NUL and traverse rest of src */
+	if (n == 0) {
+		if (siz != 0)
+			*d = '\0';	      /* NUL-terminate dst */
+		while (*s++)
+			;
+	}
 
-        return(s - src - 1);    /* count does not include NUL */
+	return(s - src - 1);    /* count does not include NUL */
 }
 #endif
 
-int fake_getnameinfo(const struct sockaddr *sa, size_t salen, char *host, 
-                size_t hostlen, char *serv, size_t servlen, int flags)
+int fake_getnameinfo(const struct sockaddr *sa, size_t salen, char *host,
+	size_t hostlen, char *serv, size_t servlen, int flags)
 {
 	struct sockaddr_in *sin = (struct sockaddr_in *)sa;
 	struct hostent *hp;
 	char tmpserv[16];
+	(void)salen;
 
 	if (sa->sa_family != AF_UNSPEC && sa->sa_family != AF_INET)
 		return (EAI_FAMILY);
@@ -84,7 +85,7 @@ int fake_getnameinfo(const struct sockaddr *sa, size_t salen, char *host,
 
 	if (host != NULL) {
 		if (flags & NI_NUMERICHOST) {
-			int len;
+			size_t len;
 			Tcl_MutexLock(&netdbMutex);
 			len = strlcpy(host, inet_ntoa(sin->sin_addr), hostlen);
 			Tcl_MutexUnlock(&netdbMutex);
@@ -96,7 +97,7 @@ int fake_getnameinfo(const struct sockaddr *sa, size_t salen, char *host,
 		} else {
 			int ret;
 			Tcl_MutexLock(&netdbMutex);
-			hp = gethostbyaddr((char *)&sin->sin_addr, 
+			hp = gethostbyaddr((char *)&sin->sin_addr,
 			    sizeof(struct in_addr), AF_INET);
 			if (hp == NULL) {
 				ret = EAI_NODATA;
@@ -130,12 +131,12 @@ fake_gai_strerror(int err)
 	default:
 		return ("unknown/invalid error.");
 	}
-}    
+}
 #endif /* !HAVE_GAI_STRERROR */
 
 #ifndef HAVE_FREEADDRINFO
 void
-freeaddrinfo(struct addrinfo *ai)
+fake_freeaddrinfo(struct addrinfo *ai)
 {
 	struct addrinfo *next;
 
@@ -153,12 +154,12 @@ addrinfo *malloc_ai(int port, u_long addr, const struct addrinfo *hints)
 {
 	struct addrinfo *ai;
 
-	ai = malloc(sizeof(*ai) + sizeof(struct sockaddr_in));
+	ai = (struct addrinfo *)malloc(sizeof(*ai) + sizeof(struct sockaddr_in));
 	if (ai == NULL)
 		return (NULL);
-	
+
 	memset(ai, '\0', sizeof(*ai) + sizeof(struct sockaddr_in));
-	
+
 	ai->ai_addr = (struct sockaddr *)(ai + 1);
 	/* XXX -- ssh doesn't use sa_len */
 	ai->ai_addrlen = sizeof(struct sockaddr_in);
@@ -166,7 +167,7 @@ addrinfo *malloc_ai(int port, u_long addr, const struct addrinfo *hints)
 
 	((struct sockaddr_in *)(ai)->ai_addr)->sin_port = port;
 	((struct sockaddr_in *)(ai)->ai_addr)->sin_addr.s_addr = addr;
-	
+
 	/* XXX: the following is not generally correct, but does what we want */
 	if (hints->ai_socktype)
 		ai->ai_socktype = hints->ai_socktype;
@@ -180,7 +181,7 @@ addrinfo *malloc_ai(int port, u_long addr, const struct addrinfo *hints)
 }
 
 int
-fake_getaddrinfo(const char *hostname, const char *servname, 
+fake_getaddrinfo(const char *hostname, const char *servname,
     const struct addrinfo *hints, struct addrinfo **res)
 {
 	struct hostent *hp;
@@ -199,7 +200,7 @@ fake_getaddrinfo(const char *hostname, const char *servname,
 
 		port = strtol(servname, &cp, 10);
 		if (port > 0 && port <= 65535 && *cp == '\0')
-			port = htons(port);
+			port = htons((unsigned short)port);
 		else if ((sp = getservbyname(servname, NULL)) != NULL)
 			port = sp->s_port;
 		else
@@ -211,29 +212,29 @@ fake_getaddrinfo(const char *hostname, const char *servname,
 		if (hostname && inet_aton(hostname, &in) != 0)
 			addr = in.s_addr;
 		*res = malloc_ai(port, addr, hints);
-		if (*res == NULL) 
+		if (*res == NULL)
 			return (EAI_MEMORY);
 		return (0);
 	}
-		
+
 	if (!hostname) {
-		*res = malloc_ai(port, htonl(0x7f000001), hints);
-		if (*res == NULL) 
+		*res = malloc_ai(port, htonl(0x7F000001), hints);
+		if (*res == NULL)
 			return (EAI_MEMORY);
 		return (0);
 	}
-	
+
 	if (inet_aton(hostname, &in)) {
 		*res = malloc_ai(port, in.s_addr, hints);
-		if (*res == NULL) 
+		if (*res == NULL)
 			return (EAI_MEMORY);
 		return (0);
 	}
-	
+
 	/* Don't try DNS if AI_NUMERICHOST is set */
 	if (hints && hints->ai_flags & AI_NUMERICHOST)
 		return (EAI_NONAME);
-	
+
 	Tcl_MutexLock(&netdbMutex);
 	hp = gethostbyname(hostname);
 	if (hp && hp->h_name && hp->h_name[0] && hp->h_addr_list[0]) {
