@@ -2505,6 +2505,8 @@ BinaryDecodeHex(
 	case OPT_STRICT:
 	    strict = 1;
 	    break;
+	default:
+	    TCL_UNREACHABLE();
 	}
     }
 
@@ -2649,6 +2651,8 @@ BinaryEncode64(
 		wrapchar = TclGetStringFromObj(objv[i + 1], &wrapcharlen);
 	    }
 	    break;
+	default:
+	    TCL_UNREACHABLE();
 	}
     }
     if (wrapcharlen == 0) {
@@ -2736,13 +2740,13 @@ BinaryEncodeUu(
 {
     Tcl_Obj *resultObj;
     unsigned char *data, *start, *cursor;
-    int i, bits, index;
+    int i, bits;
     unsigned int n;
     int lineLength = 61;
     const unsigned char SingleNewline[] = { UCHAR('\n') };
     const unsigned char *wrapchar = SingleNewline;
     Tcl_Size j, rawLength, offset, count = 0, wrapcharlen = sizeof(SingleNewline);
-    enum { OPT_MAXLEN, OPT_WRAPCHAR };
+    enum { OPT_MAXLEN, OPT_WRAPCHAR } index;
     static const char *const optStrings[] = { "-maxlen", "-wrapchar", NULL };
 
     if (objc < 2 || objc % 2 != 0) {
@@ -2773,35 +2777,33 @@ BinaryEncodeUu(
 	case OPT_WRAPCHAR:
 	    wrapchar = (const unsigned char *)TclGetStringFromObj(
 		    objv[i + 1], &wrapcharlen);
-	    {
-		const unsigned char *p = wrapchar;
-		Tcl_Size numBytes = wrapcharlen;
+	    const unsigned char *p = wrapchar;
+	    Tcl_Size numBytes = wrapcharlen;
 
-		while (numBytes) {
-		    switch (*p) {
-			case '\t':
-			case '\v':
-			case '\f':
-			case '\r':
-			    p++; numBytes--;
-			    continue;
-			case '\n':
-			    numBytes--;
-			    break;
-			default:
-			badwrap:
-			    Tcl_SetObjResult(interp, Tcl_NewStringObj(
-				    "invalid wrapchar; will defeat decoding",
-				    -1));
-			    Tcl_SetErrorCode(interp, "TCL", "BINARY",
-				    "ENCODE", "WRAPCHAR", (char *)NULL);
-			    return TCL_ERROR;
-		    }
-		}
-		if (numBytes) {
+	    while (numBytes) {
+		switch (*p) {
+		case '\t':
+		case '\v':
+		case '\f':
+		case '\r':
+		    p++;
+		    numBytes--;
+		    continue;
+		case '\n':
+		    numBytes--;
+		    break;
+		default:
 		    goto badwrap;
 		}
 	    }
+	    if (numBytes) {
+	    badwrap:
+		Tcl_SetObjResult(interp, Tcl_NewStringObj(
+			"invalid wrapchar; will defeat decoding", -1));
+		Tcl_SetErrorCode(interp, "TCL", "BINARY",
+			"ENCODE", "WRAPCHAR", (char *)NULL);
+		return TCL_ERROR;
+	    }
 	    break;
 	}
     }
@@ -2909,6 +2911,8 @@ BinaryDecodeUu(
 	case OPT_STRICT:
 	    strict = 1;
 	    break;
+	default:
+	    TCL_UNREACHABLE();
 	}
     }
 
@@ -3084,6 +3088,8 @@ BinaryDecode64(
 	case OPT_STRICT:
 	    strict = 1;
 	    break;
+	default:
+	    TCL_UNREACHABLE();
 	}
     }
 
