@@ -565,7 +565,10 @@ ClockFmtScnStorageAllocProc(
 	allocsize -= sizeof(hPtr->key);
     }
 
-    fss = (ClockFmtScnStorage *)Tcl_Alloc(allocsize);
+    fss = (ClockFmtScnStorage *)Tcl_AttemptAlloc(allocsize);
+    if (!fss) {
+	return NULL;
+    }
 
     /* initialize */
     memset(fss, 0, sizeof(*fss));
@@ -1024,7 +1027,7 @@ static const char *
 FindTokenBegin(
     const char *p,
     const char *end,
-    ClockScanToken *tok,
+    const ClockScanToken *tok,
     int flags)
 {
     if (p < end) {
@@ -1037,10 +1040,14 @@ FindTokenBegin(
 	    if (!(flags & CLF_STRICT)) {
 		/* should match at least one digit or space */
 		while (!isdigit(UCHAR(*p)) && !isspace(UCHAR(*p)) &&
-			(p = Tcl_UtfNext(p)) < end) {}
+			(p = Tcl_UtfNext(p)) < end) {
+		    // Empty
+		}
 	    } else {
 		/* should match at least one digit */
-		while (!isdigit(UCHAR(*p)) && (p = Tcl_UtfNext(p)) < end) {}
+		while (!isdigit(UCHAR(*p)) && (p = Tcl_UtfNext(p)) < end) {
+		    // Empty
+		}
 	    }
 	    return p;
 
@@ -1049,19 +1056,25 @@ FindTokenBegin(
 	    goto findChar;
 
 	case CTOKT_SPACE:
-	    while (!isspace(UCHAR(*p)) && (p = Tcl_UtfNext(p)) < end) {}
+	    while (!isspace(UCHAR(*p)) && (p = Tcl_UtfNext(p)) < end) {
+		// Empty
+	    }
 	    return p;
 
 	case CTOKT_CHAR:
 	    c = *((char *)tok->map->data);
-findChar:
+	findChar:
 	    if (!(flags & CLF_STRICT)) {
 		/* should match the char or space */
 		while (*p != c && !isspace(UCHAR(*p)) &&
-			(p = Tcl_UtfNext(p)) < end) {}
+			(p = Tcl_UtfNext(p)) < end) {
+		    // Empty
+		}
 	    } else {
 		/* should match the char */
-		while (*p != c && (p = Tcl_UtfNext(p)) < end) {}
+		while (*p != c && (p = Tcl_UtfNext(p)) < end) {
+		    // Empty
+		}
 	    }
 	    return p;
 	}
@@ -1089,7 +1102,7 @@ static void
 DetermineGreedySearchLen(
     ClockFmtScnCmdArgs *opts,
     DateInfo *info,
-    ClockScanToken *tok,
+    const ClockScanToken *tok,
     int *minLenPtr,
     int *maxLenPtr)
 {
@@ -1141,7 +1154,7 @@ DetermineGreedySearchLen(
     /* try to get max length more precise for greedy match,
      * check the next ahead token available there */
     if (minLen < maxLen && tok->lookAhTok) {
-	ClockScanToken *laTok = tok + tok->lookAhTok + 1;
+	const ClockScanToken *laTok = tok + tok->lookAhTok + 1;
 
 	p = yyInput + maxLen;
 	/* regards all possible spaces here (because they are optional) */
@@ -1155,7 +1168,7 @@ DetermineGreedySearchLen(
 	    /* try to find laTok between [lookAhMin, lookAhMax] */
 	    while (minLen < maxLen) {
 		const char *f = FindTokenBegin(p, end, laTok,
-				    TCL_CLOCK_FULL_COMPAT ? opts->flags : CLF_STRICT);
+			TCL_CLOCK_FULL_COMPAT ? opts->flags : CLF_STRICT);
 		/* if found (not below lookAhMax) */
 		if (f < end) {
 		    break;
@@ -1491,7 +1504,7 @@ StaticListSearch(
 
 static inline const char *
 FindWordEnd(
-    ClockScanToken *tok,
+    const ClockScanToken *tok,
     const char *p,
     const char *end)
 {
@@ -1516,7 +1529,7 @@ static int
 ClockScnToken_Month_Proc(
     ClockFmtScnCmdArgs *opts,
     DateInfo *info,
-    ClockScanToken *tok)
+    const ClockScanToken *tok)
 {
 #if 0
 /* currently unused, test purposes only */
@@ -1566,7 +1579,7 @@ static int
 ClockScnToken_DayOfWeek_Proc(
     ClockFmtScnCmdArgs *opts,
     DateInfo *info,
-    ClockScanToken *tok)
+    const ClockScanToken *tok)
 {
     static int dowKeys[] = {MCLIT_DAYS_OF_WEEK_ABBREV, MCLIT_DAYS_OF_WEEK_FULL, 0};
 
@@ -1640,7 +1653,7 @@ static int
 ClockScnToken_amPmInd_Proc(
     ClockFmtScnCmdArgs *opts,
     DateInfo *info,
-    ClockScanToken *tok)
+    const ClockScanToken *tok)
 {
     int ret, val;
     int minLen, maxLen;
@@ -1673,7 +1686,7 @@ static int
 ClockScnToken_LocaleERA_Proc(
     ClockFmtScnCmdArgs *opts,
     DateInfo *info,
-    ClockScanToken *tok)
+    const ClockScanToken *tok)
 {
     ClockClientData *dataPtr = opts->dataPtr;
 
@@ -1712,7 +1725,7 @@ static int
 ClockScnToken_LocaleListMatcher_Proc(
     ClockFmtScnCmdArgs *opts,
     DateInfo *info,
-    ClockScanToken *tok)
+    const ClockScanToken *tok)
 {
     int ret, val;
     int minLen, maxLen;
@@ -1743,7 +1756,7 @@ static int
 ClockScnToken_JDN_Proc(
     ClockFmtScnCmdArgs *opts,
     DateInfo *info,
-    ClockScanToken *tok)
+    const ClockScanToken *tok)
 {
     int minLen, maxLen;
     const char *p = yyInput, *end, *s;
@@ -1814,7 +1827,7 @@ static int
 ClockScnToken_TimeZone_Proc(
     ClockFmtScnCmdArgs *opts,
     DateInfo *info,
-    ClockScanToken *tok)
+    const ClockScanToken *tok)
 {
     int minLen, maxLen;
     int len = 0;
@@ -1906,7 +1919,7 @@ static int
 ClockScnToken_StarDate_Proc(
     ClockFmtScnCmdArgs *opts,
     DateInfo *info,
-    ClockScanToken *tok)
+    const ClockScanToken *tok)
 {
     int minLen, maxLen;
     const char *p = yyInput, *end, *s;
@@ -2300,8 +2313,7 @@ ClockGetOrParseScanFormat(
 		    tokCnt++;
 		    continue;
 		}
-	      word_tok:
-		{
+	    word_tok: {
 		/* try continue with previous word token */
 		ClockScanToken *wordTok = tok - 1;
 
@@ -2324,9 +2336,9 @@ ClockGetOrParseScanFormat(
 		    AllocTokenInChain(tok, scnTok, fss->scnTokC, ClockScanToken *);
 		    tokCnt++;
 		}
-		}
 		break;
 	    }
+	    }
 	}
 
 	/* calculate end distance value for each tokens */
@@ -2373,8 +2385,8 @@ ClockScan(
     ClockFmtScnCmdArgs *opts)	/* Command options */
 {
     ClockClientData *dataPtr = opts->dataPtr;
-    ClockFmtScnStorage *fss;
-    ClockScanToken *tok;
+    const ClockFmtScnStorage *fss;
+    const ClockScanToken *tok;
     const ClockScanTokenMap *map;
     const char *p, *x, *end;
     unsigned short flags = 0;
@@ -2554,6 +2566,8 @@ ClockScan(
 	    }
 	    p++;
 	    break;
+	default:
+	    TCL_UNREACHABLE();
 	}
     }
     /* check end was reached */
@@ -2602,7 +2616,7 @@ ClockScan(
 		case (CLF_DAYOFYEAR | CLF_DAYOFMONTH):
 		    /* miss month: ddd over dd (without month) */
 		    flags &= ~CLF_DAYOFMONTH;
-		    /* fallthrough */
+		    TCL_FALLTHROUGH();
 		case CLF_DAYOFYEAR:
 		    /* ddd over naked weekday */
 		    if (!(flags & CLF_ISO8601YEAR)) {
@@ -3343,8 +3357,7 @@ ClockGetOrParseFmtFormat(
 		continue;
 	    }
 	    default:
-	      word_tok:
-		{
+	    word_tok: {
 		/* try continue with previous word token */
 		ClockFormatToken *wordTok = tok - 1;
 
@@ -3363,9 +3376,9 @@ ClockGetOrParseFmtFormat(
 		    AllocTokenInChain(tok, fmtTok, fss->fmtTokC, ClockFormatToken *);
 		    tokCnt++;
 		}
-		}
 		break;
 	    }
+	    }
 	}
 
 	/* correct count of real used tokens and free mem if desired
