@@ -6,8 +6,8 @@
  *	is primarily responsible for keeping the "env" arrays in sync with the
  *	system environment variables.
  *
- * Copyright (c) 1991-1994 The Regents of the University of California.
- * Copyright (c) 1994-1998 Sun Microsystems, Inc.
+ * Copyright © 1991-1994 The Regents of the University of California.
+ * Copyright © 1994-1998 Sun Microsystems, Inc.
  *
  * See the file "license.terms" for information on usage and redistribution of
  * this file, and for a DISCLAIMER OF ALL WARRANTIES.
@@ -19,30 +19,29 @@ TCL_DECLARE_MUTEX(envMutex)	/* To serialize access to environ. */
 
 #if defined(_WIN32)
 #  define tenviron _wenviron
-#  define tenviron2utfdstr(tenvstr, len, dstr) \
-		Tcl_WinTCharToUtf((TCHAR *)tenvstr, len, dstr)
-#  define utf2tenvirondstr(str, len, dstr) \
-		(const WCHAR *)Tcl_WinUtfToTChar(str, len, dstr)
+#  define tenviron2utfdstr(str, dsPtr) (Tcl_DStringInit(dsPtr), \
+	(char *)Tcl_Char16ToUtfDString((const unsigned short *)(str), -1, (dsPtr)))
+#  define utf2tenvirondstr(str, dsPtr) (Tcl_DStringInit(dsPtr), \
+	(const WCHAR *)Tcl_UtfToChar16DString((str), -1, (dsPtr)))
 #  define techar WCHAR
 #  ifdef USE_PUTENV
 #    define putenv(env) _wputenv((const wchar_t *)env)
 #  endif
 #else
 #  define tenviron environ
-#  define tenviron2utfdstr(tenvstr, len, dstr) \
-		Tcl_ExternalToUtfDString(NULL, tenvstr, len, dstr)
-#  define utf2tenvirondstr(str, len, dstr) \
-		Tcl_UtfToExternalDString(NULL, str, len, dstr)
+#  define tenviron2utfdstr(str, dsPtr) \
+	Tcl_ExternalToUtfDString(NULL, str, -1, dsPtr)
+#  define utf2tenvirondstr(str, dsPtr) \
+	Tcl_UtfToExternalDString(NULL, str, -1, dsPtr)
 #  define techar char
 #endif
 
-
 /* MODULE_SCOPE */
 size_t TclEnvEpoch = 0;	/* Epoch of the tcl environment
 				 * (if changed with tcl-env). */
 
 static struct {
-    int cacheSize;		/* Number of env strings in cache. */
+    Tcl_Size cacheSize;		/* Number of env strings in cache. */
     char **cache;		/* Array containing all of the environment
 				 * strings that Tcl has allocated. */
 #ifndef USE_PUTENV
@@ -50,7 +49,7 @@ static struct {
 				 * need to track this in case another
 				 * subsystem swaps around the environ array
 				 * like we do. */
-    int ourEnvironSize;		/* Non-zero means that the environ array was
+    Tcl_Size ourEnvironSize;	/* Non-zero means that the environ array was
 				 * malloced and has this many total entries
 				 * allocated to it (not all may be in use at
 				 * once). Zero means that the environment
@@ -64,7 +63,7 @@ static struct {
  * Declarations for local functions defined in this file:
  */
 
-static char *		EnvTraceProc(ClientData clientData, Tcl_Interp *interp,
+static char *		EnvTraceProc(void *clientData, Tcl_Interp *interp,
 			    const char *name1, const char *name2, int flags);
 static void		ReplaceString(const char *oldStr, char *newStr);
 MODULE_SCOPE void	TclSetEnv(const char *name, const char *value);
@@ -159,7 +158,11 @@ TclSetupEnv(
 	    const char *p1;
 	    char *p2;
 
-	    p1 = tenviron2utfdstr(tenviron[i], -1, &envString);
+	    p1 = tenviron2utfdstr(tenviron[i], &envString);
+	    if (p1 == NULL) {
+		/* Ignore what cannot be decoded (should not happen) */
+		continue;
+	    }
 	    p2 = (char *)strchr(p1, '=');
 	    if (p2 == NULL) {
 		/*
@@ -253,8 +256,8 @@ TclSetEnv(
     const char *value)		/* New value for variable (UTF-8). */
 {
     Tcl_DString envString;
-    unsigned nameLength, valueLength;
-    int index, length;
+    Tcl_Size nameLength, valueLength;
+    Tcl_Size index, length;
     char *p, *oldValue;
     const techar *p2;
 
@@ -267,7 +270,7 @@ TclSetEnv(
     Tcl_MutexLock(&envMutex);
     index = TclpFindVariable(name, &length);
 
-    if (index == -1) {
+    if (index == TCL_INDEX_NONE) {
 #ifndef USE_PUTENV
 	/*
 	 * We need to handle the case where the environment may be changed
@@ -276,11 +279,11 @@ TclSetEnv(
 	 */
 
 	if ((env.ourEnviron != tenviron) || (length+2 > env.ourEnvironSize)) {
-	    techar **newEnviron = (techar **)ckalloc((length + 5) * sizeof(techar *));
+	    techar **newEnviron = (techar **)Tcl_Alloc((length + 5) * sizeof(techar *));
 
 	    memcpy(newEnviron, tenviron, length * sizeof(techar *));
 	    if ((env.ourEnvironSize != 0) && (env.ourEnviron != NULL)) {
-		ckfree(env.ourEnviron);
+		Tcl_Free(env.ourEnviron);
 	    }
 	    tenviron = (env.ourEnviron = newEnviron);
 	    env.ourEnvironSize = length + 5;
@@ -301,9 +304,9 @@ TclSetEnv(
 	 * interpreters.
 	 */
 
-	oldEnv = tenviron2utfdstr(tenviron[index], -1, &envString);
-	if (strcmp(value, oldEnv + (length + 1)) == 0) {
-	    Tcl_DStringFree(&envString);
+	oldEnv = tenviron2utfdstr(tenviron[index], &envString);
+	if (oldEnv == NULL || strcmp(value, oldEnv + (length + 1)) == 0) {
+	    Tcl_DStringFree(&envString); /* OK even if oldEnv is NULL */
 	    Tcl_MutexUnlock(&envMutex);
 	    return;
 	}
@@ -320,17 +323,23 @@ TclSetEnv(
      */
 
     valueLength = strlen(value);
-    p = (char *)ckalloc(nameLength + valueLength + 2);
+    p = (char *)Tcl_Alloc(nameLength + valueLength + 2);
     memcpy(p, name, nameLength);
     p[nameLength] = '=';
     memcpy(p+nameLength+1, value, valueLength+1);
-    p2 = utf2tenvirondstr(p, -1, &envString);
+    p2 = utf2tenvirondstr(p, &envString);
+    if (p2 == NULL) {
+	/* No way to signal error from here :-( but should not happen */
+	Tcl_Free(p);
+	Tcl_MutexUnlock(&envMutex);
+	return;
+    }
 
     /*
      * Copy the native string to heap memory.
      */
 
-    p = (char *)ckrealloc(p, Tcl_DStringLength(&envString) + tNTL);
+    p = (char *)Tcl_Realloc(p, Tcl_DStringLength(&envString) + tNTL);
     memcpy(p, p2, Tcl_DStringLength(&envString) + tNTL);
     Tcl_DStringFree(&envString);
 
@@ -351,7 +360,7 @@ TclSetEnv(
      * string in the cache.
      */
 
-    if ((index != -1) && (tenviron[index] == (techar *)p)) {
+    if ((index != TCL_INDEX_NONE) && (tenviron[index] == (techar *)p)) {
 	ReplaceString(oldValue, p);
 #ifdef HAVE_PUTENV_THAT_COPIES
     } else {
@@ -359,20 +368,11 @@ TclSetEnv(
 	 * This putenv() copies instead of taking ownership.
 	 */
 
-	ckfree(p);
+	Tcl_Free(p);
 #endif /* HAVE_PUTENV_THAT_COPIES */
     }
 
     Tcl_MutexUnlock(&envMutex);
-
-    if (!strcmp(name, "HOME")) {
-	/*
-	 * If the user's home directory has changed, we must invalidate the
-	 * filesystem cache, because '~' expansions will now be incorrect.
-	 */
-
-	Tcl_FSMountsChanged(NULL);
-    }
 }
 
 /*
@@ -411,11 +411,11 @@ Tcl_PutEnv(
     }
 
     /*
-     * First convert the native string to UTF. Then separate the string into
+     * First convert the native string to Utf. Then separate the string into
      * name and value parts, and call TclSetEnv to do all of the real work.
      */
 
-    name = Tcl_ExternalToUtfDString(NULL, assignment, -1, &nameString);
+    name = Tcl_ExternalToUtfDString(NULL, assignment, TCL_INDEX_NONE, &nameString);
     value = (char *)strchr(name, '=');
 
     if ((value != NULL) && (value != name)) {
@@ -462,8 +462,7 @@ TclUnsetEnv(
     const char *name)		/* Name of variable to remove (UTF-8). */
 {
     char *oldValue;
-    int length;
-    int index;
+    Tcl_Size length, index;
 #ifdef USE_PUTENV_FOR_UNSET
     Tcl_DString envString;
     char *string;
@@ -502,18 +501,22 @@ TclUnsetEnv(
      */
 
 #if defined(_WIN32)
-    string = (char *)ckalloc(length + 2);
+    string = (char *)Tcl_Alloc(length + 2);
     memcpy(string, name, length);
     string[length] = '=';
     string[length+1] = '\0';
 #else
-    string = (char *)ckalloc(length + 1);
+    string = (char *)Tcl_Alloc(length + 1);
     memcpy(string, name, length);
     string[length] = '\0';
 #endif /* _WIN32 */
 
-    utf2tenvirondstr(string, -1, &envString);
-    string = (char *)ckrealloc(string, Tcl_DStringLength(&envString) + tNTL);
+    if (utf2tenvirondstr(string, &envString) == NULL) {
+	/* Should not happen except memory alloc fail. */
+	Tcl_MutexUnlock(&envMutex);
+	return;
+    }
+    string = (char *)Tcl_Realloc(string, Tcl_DStringLength(&envString) + tNTL);
     memcpy(string, Tcl_DStringValue(&envString),
 	    Tcl_DStringLength(&envString) + tNTL);
     Tcl_DStringFree(&envString);
@@ -534,7 +537,7 @@ TclUnsetEnv(
 	 * This putenv() copies instead of taking ownership.
 	 */
 
-	ckfree(string);
+	Tcl_Free(string);
 #endif /* HAVE_PUTENV_THAT_COPIES */
     }
 #else /* !USE_PUTENV_FOR_UNSET */
@@ -578,7 +581,7 @@ TclGetEnv(
 				 * value of the environment variable is
 				 * stored. */
 {
-    int length, index;
+    Tcl_Size length, index;
     const char *result;
 
     Tcl_MutexLock(&envMutex);
@@ -587,17 +590,19 @@ TclGetEnv(
     if (index != -1) {
 	Tcl_DString envStr;
 
-	result = tenviron2utfdstr(tenviron[index], -1, &envStr);
-	result += length;
-	if (*result == '=') {
-	    result++;
-	    Tcl_DStringInit(valuePtr);
-	    Tcl_DStringAppend(valuePtr, result, -1);
-	    result = Tcl_DStringValue(valuePtr);
-	} else {
-	    result = NULL;
+	result = tenviron2utfdstr(tenviron[index], &envStr);
+	if (result) {
+	    result += length;
+	    if (*result == '=') {
+		result++;
+		Tcl_DStringInit(valuePtr);
+		Tcl_DStringAppend(valuePtr, result, -1);
+		result = Tcl_DStringValue(valuePtr);
+	    } else {
+		result = NULL;
+	    }
+	    Tcl_DStringFree(&envStr);
 	}
-	Tcl_DStringFree(&envStr);
     }
     Tcl_MutexUnlock(&envMutex);
     return result;
@@ -624,10 +629,9 @@ TclGetEnv(
  *----------------------------------------------------------------------
  */
 
-	/* ARGSUSED */
 static char *
 EnvTraceProc(
-    ClientData clientData,	/* Not used. */
+    TCL_UNUSED(void *),
     Tcl_Interp *interp,		/* Interpreter whose "env" variable is being
 				 * modified. */
     const char *name1,		/* Better be "env". */
@@ -659,8 +663,19 @@ EnvTraceProc(
 
     if (flags & TCL_TRACE_WRITES) {
 	const char *value;
+	Tcl_DString ds;
 
 	value = Tcl_GetVar2(interp, "env", name2, TCL_GLOBAL_ONLY);
+	Tcl_DStringInit(&ds);
+	if (Tcl_UtfToExternalDStringEx(NULL, TCLFSENCODING, name2, -1, 0, &ds, NULL) != TCL_OK) {
+	    Tcl_DStringFree(&ds);
+	    return (char *) "encoding error";
+	}
+	if (Tcl_UtfToExternalDStringEx(NULL, TCLFSENCODING, value, -1, 0, &ds, NULL) != TCL_OK) {
+	    Tcl_DStringFree(&ds);
+	    return (char *) "encoding error";
+	}
+	Tcl_DStringFree(&ds);
 	TclSetEnv(name2, value);
 	TclEnvEpoch++;
     }
@@ -714,7 +729,7 @@ ReplaceString(
     const char *oldStr,		/* Old environment string. */
     char *newStr)		/* New environment string. */
 {
-    int i;
+    Tcl_Size i;
 
     /*
      * Check to see if the old value was allocated by Tcl. If so, it needs to
@@ -734,7 +749,7 @@ ReplaceString(
 	 */
 
 	if (env.cache[i]) {
-	    ckfree(env.cache[i]);
+	    Tcl_Free(env.cache[i]);
 	}
 
 	if (newStr) {
@@ -752,11 +767,11 @@ ReplaceString(
 
 	const int growth = 5;
 
-	env.cache = (char **)ckrealloc(env.cache,
+	env.cache = (char **)Tcl_Realloc(env.cache,
 		(env.cacheSize + growth) * sizeof(char *));
 	env.cache[env.cacheSize] = newStr;
 	(void) memset(env.cache+env.cacheSize+1, 0,
-		(size_t) (growth-1) * sizeof(char *));
+		(growth-1) * sizeof(char *));
 	env.cacheSize += growth;
     }
 }
@@ -793,17 +808,17 @@ TclFinalizeEnvironment(void)
 
     if (env.cache) {
 #ifdef PURIFY
-	int i;
+	Tcl_Size i;
 	for (i = 0; i < env.cacheSize; i++) {
-	    ckfree(env.cache[i]);
+	    Tcl_Free(env.cache[i]);
 	}
 #endif
-	ckfree(env.cache);
+	Tcl_Free(env.cache);
 	env.cache = NULL;
 	env.cacheSize = 0;
 #ifndef USE_PUTENV
 	if ((env.ourEnviron != NULL)) {
-	    ckfree(env.ourEnviron);
+	    Tcl_Free(env.ourEnviron);
 	    env.ourEnviron = NULL;
 	}
 	env.ourEnvironSize = 0;
