@@ -5,8 +5,8 @@
  *	integers or floating-point numbers or booleans, doing syntax checking
  *	along the way.
  *
- * Copyright (c) 1990-1993 The Regents of the University of California.
- * Copyright (c) 1994-1997 Sun Microsystems, Inc.
+ * Copyright © 1990-1993 The Regents of the University of California.
+ * Copyright © 1994-1997 Sun Microsystems, Inc.
  *
  * See the file "license.terms" for information on usage and redistribution of
  * this file, and for a DISCLAIMER OF ALL WARRANTIES.
@@ -53,7 +53,7 @@ Tcl_GetInt(
     if (obj.refCount > 1) {
 	Tcl_Panic("invalid sharing of Tcl_Obj on C stack");
     }
-    TclFreeIntRep(&obj);
+    TclFreeInternalRep(&obj);
     return code;
 }
 
@@ -97,7 +97,7 @@ Tcl_GetDouble(
     if (obj.refCount > 1) {
 	Tcl_Panic("invalid sharing of Tcl_Obj on C stack");
     }
-    TclFreeIntRep(&obj);
+    TclFreeInternalRep(&obj);
     return code;
 }
 
@@ -110,7 +110,7 @@ Tcl_GetDouble(
  *	string.
  *
  * Results:
- *	The return value is normally TCL_OK; in this case *intPtr will be set
+ *	The return value is normally TCL_OK; in this case *charPtr will be set
  *	to the 0/1 value equivalent to src. If src is improperly formed then
  *	TCL_ERROR is returned and an error message will be left in the
  *	interp's result.
@@ -121,17 +121,23 @@ Tcl_GetDouble(
  *----------------------------------------------------------------------
  */
 
+#undef Tcl_GetBool
+#undef Tcl_GetBoolFromObj
 int
-Tcl_GetBoolean(
+Tcl_GetBool(
     Tcl_Interp *interp,		/* Interpreter used for error reporting. */
     const char *src,		/* String containing one of the boolean values
 				 * 1, 0, true, false, yes, no, on, off. */
-    int *intPtr)		/* Place to store converted result, which will
+    int flags,
+    char *charPtr)		/* Place to store converted result, which will
 				 * be 0 or 1. */
 {
     Tcl_Obj obj;
     int code;
 
+    if ((src == NULL) || (*src == '\0')) {
+	return Tcl_GetBoolFromObj(interp, NULL, flags, charPtr);
+    }
     obj.refCount = 1;
     obj.bytes = (char *) src;
     obj.length = strlen(src);
@@ -142,10 +148,22 @@ Tcl_GetBoolean(
 	Tcl_Panic("invalid sharing of Tcl_Obj on C stack");
     }
     if (code == TCL_OK) {
-	*intPtr = obj.internalRep.longValue;
+	Tcl_GetBoolFromObj(NULL, &obj, flags, charPtr);
     }
     return code;
 }
+
+#undef Tcl_GetBoolean
+int
+Tcl_GetBoolean(
+    Tcl_Interp *interp,		/* Interpreter used for error reporting. */
+    const char *src,		/* String containing one of the boolean values
+				 * 1, 0, true, false, yes, no, on, off. */
+    int *intPtr)		/* Place to store converted result, which will
+				 * be 0 or 1. */
+{
+    return Tcl_GetBool(interp, src, (TCL_NULL_OK-2)&(int)sizeof(int), (char *)(void *)intPtr);
+}
 
 /*
  * Local Variables:
