@@ -2108,6 +2108,8 @@ EncodeEventMask(
 	case EVENT_WRITE:
 	    events |= TCL_WRITABLE;
 	    break;
+	default:
+	    TCL_UNREACHABLE();
 	}
 	listc --;
     }
