@@ -332,8 +332,8 @@ Tcl_OpenTcpServer(
     char portbuf[TCL_INTEGER_SPACE];
 
     TclFormatInt(portbuf, port);
-    return Tcl_OpenTcpServerEx(interp, portbuf, host, -1,
-	    TCL_TCPSERVER_REUSEADDR, acceptProc, callbackData);
+    return Tcl_OpenTcpServerEx(interp, portbuf, host, TCL_TCPSERVER_REUSEADDR,
+	    -1, acceptProc, callbackData);
 }
 
 /*
