@@ -349,10 +349,15 @@ DetectEncoding(
 	return FunctionNotAvailableError(interp);
     }
 
-    bytes = (char *) Tcl_GetBytesFromObj(interp, objPtr, &len);
+    bytes = (char *)Tcl_GetBytesFromObj(interp, objPtr, &len);
     if (bytes == NULL) {
 	return TCL_ERROR;
     }
+    if (len > INT_MAX) {
+	Tcl_SetObjResult(interp,
+		Tcl_NewStringObj("Max length supported by ICU exceeded.", TCL_INDEX_NONE));
+	return TCL_ERROR;
+    }
     UErrorCodex status = U_ZERO_ERRORZ;
 
     UCharsetDetector* csd = ucsdet_open(&status);
@@ -360,7 +365,7 @@ DetectEncoding(
 	return IcuError(interp, "Could not open charset detector", status);
     }
 
-    ucsdet_setText(csd, bytes, len, &status);
+    ucsdet_setText(csd, bytes, (int)len, &status);
     if (U_FAILURE(status)) {
 	IcuError(interp, "Could not set detection text", status);
 	ucsdet_close(csd);
@@ -569,11 +574,11 @@ IcuObjFromUCharDString(
 /*
  *------------------------------------------------------------------------
  *
- * EncodingDetectObjCmd --
+ * IcuDetectObjCmd --
  *
- *	Implements the Tcl command EncodingDetect.
- *	  encdetect - returns names of all detectable encodings
- *	  encdetect BYTES ?-all? - return detected encoding(s)
+ *	Implements the Tcl command ::tcl::unsupported::icu::detect.
+ *	  ::tcl::unsupported::icu::detect - returns names of all detectable encodings
+ *	  ::tcl::unsupported::icu::detect BYTES ?-all? - return detected encoding(s)
  *
  * Results:
  *	TCL_OK    - Success.
@@ -743,10 +748,10 @@ IcuConverterAliasesObjCmd(
 static int
 IcuConverttoDString(
     Tcl_Interp *interp,
-    Tcl_DString *dsInPtr,  /* Input UTF16 */
+    Tcl_DString *dsInPtr,	/* Input UTF16 */
     const char *icuEncName,
     int strict,
-    Tcl_DString *dsOutPtr) /* Output encoded string. */
+    Tcl_DString *dsOutPtr)	/* Output encoded string. */
 {
     if (ucnv_open == NULL || ucnv_close == NULL ||
 	ucnv_fromUChars == NULL || UCNV_FROM_U_CALLBACK_STOP == NULL) {
@@ -771,7 +776,7 @@ IcuConverttoDString(
     Tcl_Size utf16len = Tcl_DStringLength(dsInPtr) / sizeof(UCharx);
     Tcl_Size dstLen, dstCapacity;
     if (utf16len > INT_MAX) {
-	Tcl_SetObjResult( interp,
+	Tcl_SetObjResult(interp,
 		Tcl_NewStringObj("Max length supported by ICU exceeded.", TCL_INDEX_NONE));
 	return TCL_ERROR;
     }
@@ -779,21 +784,21 @@ IcuConverttoDString(
     dstCapacity = utf16len;
     Tcl_DStringInit(dsOutPtr);
     Tcl_DStringSetLength(dsOutPtr, dstCapacity);
-    dstLen = ucnv_fromUChars(ucnvPtr, Tcl_DStringValue(dsOutPtr), dstCapacity,
-			     utf16, utf16len, &status);
+    dstLen = ucnv_fromUChars(ucnvPtr, Tcl_DStringValue(dsOutPtr), (int)dstCapacity,
+			     utf16, (int)utf16len, &status);
     if (U_FAILURE(status)) {
 	switch (status) {
 	case U_STRING_NOT_TERMINATED_WARNING:
 	    break; /* We don't care */
 	case U_BUFFER_OVERFLOW_ERROR:
-	    Tcl_DStringSetLength(dsOutPtr, dstLen);
+	    Tcl_DStringSetLength(dsOutPtr, (int)dstLen);
 	    status = U_ZERO_ERRORZ; /* Reset before call */
-	    dstLen = ucnv_fromUChars(ucnvPtr, Tcl_DStringValue(dsOutPtr), dstLen,
-				     utf16, utf16len, &status);
+	    dstLen = ucnv_fromUChars(ucnvPtr, Tcl_DStringValue(dsOutPtr), (int)dstLen,
+				     utf16, (int)utf16len, &status);
 	    if (U_SUCCESS(status)) {
 		break;
 	    }
-	    /* FALLTHRU */
+	    TCL_FALLTHROUGH();
 	default:
 	    Tcl_DStringFree(dsOutPtr);
 	    ucnv_close(ucnvPtr);
@@ -827,7 +832,7 @@ IcuBytesToUCharDString(
     Tcl_Size nbytes,
     const char *icuEncName,
     int strict,
-    Tcl_DString *dsOutPtr) /* Output UChar string. */
+    Tcl_DString *dsOutPtr)	/* Output UChar string. */
 {
     if (ucnv_open == NULL || ucnv_close == NULL ||
 	ucnv_toUChars == NULL || UCNV_TO_U_CALLBACK_STOP == NULL) {
@@ -855,11 +860,11 @@ IcuBytesToUCharDString(
     }
 
     int dstLen;
-    int dstCapacity = (int) nbytes; /* In UChar's  */
+    int dstCapacity = (int)nbytes; /* In UChar's */
     Tcl_DStringInit(dsOutPtr);
     Tcl_DStringSetLength(dsOutPtr, dstCapacity);
     dstLen = ucnv_toUChars(ucnvPtr, (UCharx *)Tcl_DStringValue(dsOutPtr), dstCapacity,
-			   (const char *)bytes, nbytes, &status);
+			   (const char *)bytes, (int)nbytes, &status);
     if (U_FAILURE(status)) {
 	switch (status) {
 	case U_STRING_NOT_TERMINATED_WARNING:
@@ -869,11 +874,11 @@ IcuBytesToUCharDString(
 	    Tcl_DStringSetLength(dsOutPtr, dstCapacity);
 	    status = U_ZERO_ERRORZ; /* Reset before call */
 	    dstLen = ucnv_toUChars(ucnvPtr, (UCharx *)Tcl_DStringValue(dsOutPtr), dstCapacity,
-				   (const char *)bytes, nbytes, &status);
+				   (const char *)bytes, (int)nbytes, &status);
 	    if (U_SUCCESS(status)) {
 		break;
 	    }
-	    /* FALLTHRU */
+	    TCL_FALLTHROUGH();
 	default:
 	    Tcl_DStringFree(dsOutPtr);
 	    ucnv_close(ucnvPtr);
@@ -905,9 +910,9 @@ IcuBytesToUCharDString(
 static int
 IcuNormalizeUCharDString(
     Tcl_Interp *interp,
-    Tcl_DString *dsInPtr, /* Input UTF16 */
+    Tcl_DString *dsInPtr,	/* Input UTF16 */
     NormalizationMode mode,
-    Tcl_DString *dsOutPtr) /* Output normalized UTF16. */
+    Tcl_DString *dsOutPtr)	/* Output normalized UTF16. */
 {
     typedef UNormalizer2 *(*normFn)(UErrorCodex *);
     normFn fn = NULL;
@@ -953,7 +958,7 @@ IcuNormalizeUCharDString(
     normPtr = (UCharx *) Tcl_DStringValue(dsOutPtr);
 
     normLen = unorm2_normalize(
-	normalizer, utf16, utf16len, normPtr, utf16len, &status);
+	normalizer, utf16, (int)utf16len, normPtr, (int)utf16len, &status);
     if (U_FAILURE(status)) {
 	switch (status) {
 	case U_STRING_NOT_TERMINATED_WARNING:
@@ -965,11 +970,11 @@ IcuNormalizeUCharDString(
 	    normPtr = (UCharx *) Tcl_DStringValue(dsOutPtr);
 	    status = U_ZERO_ERRORZ; /* Need to clear error! */
 	    normLen = unorm2_normalize(
-		normalizer, utf16, utf16len, normPtr, normLen, &status);
+		normalizer, utf16, (int)utf16len, normPtr, normLen, &status);
 	    if (U_SUCCESS(status)) {
 		break;
 	    }
-	    /* FALLTHRU */
+	    TCL_FALLTHROUGH();
 	default:
 	    Tcl_DStringFree(dsOutPtr);
 	    return IcuError(interp, "String normalization failed", status);
@@ -1020,8 +1025,7 @@ static int IcuParseConvertOptions(
 	    if (!strcmp(s, "replace")) {
 		strict = 0;
 	    } else if (strcmp(s, "strict")) {
-		Tcl_SetObjResult(
-		    interp,
+		Tcl_SetObjResult(interp,
 		    Tcl_ObjPrintf("Invalid value \"%s\" supplied for option"
 			 " \"-profile\". Must be \"strict\" or \"replace\".",
 			 s));
@@ -1033,6 +1037,8 @@ static int IcuParseConvertOptions(
 	    Tcl_SetObjResult(interp,
 		    Tcl_NewStringObj("Option -failindex not implemented.", TCL_INDEX_NONE));
 	    return TCL_ERROR;
+	default:
+	    TCL_UNREACHABLE();
 	}
     }
     *strictPtr = strict;
@@ -1060,9 +1066,9 @@ static int IcuParseConvertOptions(
 static int
 IcuConvertfromObjCmd(
     TCL_UNUSED(void *),
-    Tcl_Interp *interp,    /* Current interpreter. */
-    int objc,              /* Number of arguments. */
-    Tcl_Obj *const objv[]) /* Argument objects. */
+    Tcl_Interp *interp,		/* Current interpreter. */
+    int objc,			/* Number of arguments. */
+    Tcl_Obj *const objv[])	/* Argument objects. */
 {
     int strict;
     Tcl_Obj *failindexVar;
@@ -1111,9 +1117,9 @@ IcuConvertfromObjCmd(
 static int
 IcuConverttoObjCmd(
     TCL_UNUSED(void *),
-    Tcl_Interp *interp,    /* Current interpreter. */
-    int objc,              /* Number of arguments. */
-    Tcl_Obj *const objv[]) /* Argument objects. */
+    Tcl_Interp *interp,		/* Current interpreter. */
+    int objc,			/* Number of arguments. */
+    Tcl_Obj *const objv[])	/* Argument objects. */
 {
     int strict;
     Tcl_Obj *failindexVar;
@@ -1156,9 +1162,9 @@ IcuConverttoObjCmd(
 static int
 IcuNormalizeObjCmd(
     TCL_UNUSED(void *),
-    Tcl_Interp *interp,    /* Current interpreter. */
-    int objc,              /* Number of arguments. */
-    Tcl_Obj *const objv[]) /* Argument objects. */
+    Tcl_Interp *interp,		/* Current interpreter. */
+    int objc,			/* Number of arguments. */
+    Tcl_Obj *const objv[])	/* Argument objects. */
 {
     static const char *optNames[] = {"-profile", "-mode", NULL};
     enum { OPT_PROFILE, OPT_MODE } opt;
@@ -1189,8 +1195,7 @@ IcuNormalizeObjCmd(
 	    if (!strcmp(s, "replace")) {
 		strict = 0;
 	    } else if (strcmp(s, "strict")) {
-		Tcl_SetObjResult(
-		    interp,
+		Tcl_SetObjResult(interp,
 		    Tcl_ObjPrintf("Invalid value \"%s\" supplied for option \"-profile\". Must be "
 				  "\"strict\" or \"replace\".",
 				  s));
@@ -1202,6 +1207,8 @@ IcuNormalizeObjCmd(
 		return TCL_ERROR;
 	    }
 	    break;
+	default:
+	    TCL_UNREACHABLE();
 	}
     }
 
@@ -1271,10 +1278,9 @@ TclIcuCleanup(
  */
 static void *
 IcuFindSymbol(
-    Tcl_LoadHandle loadH, /* Handle to shared library containing symbol */
-    const char *name,     /* Name of function */
-    const char *suffix    /* Suffix that may be present */
-)
+    Tcl_LoadHandle loadH,	/* Handle to shared library containing symbol */
+    const char *name,		/* Name of function */
+    const char *suffix)		/* Suffix that may be present */
 {
     /*
      * ICU symbols may have a version suffix depending on how it was built.
@@ -1357,7 +1363,8 @@ TclIcuInit(
 	/* Going back down to ICU version 60 */
 	while ((icu_fns.libs[0] == NULL) && (icuversion[1] >= '6')) {
 	    if (--icuversion[2] < '0') {
-		icuversion[1]--; icuversion[2] = '9';
+		icuversion[1]--;
+		icuversion[2] = '9';
 	    }
 #if defined(__CYGWIN__)
 	    i = 2;
@@ -1434,7 +1441,7 @@ TclIcuInit(
 	}
 #endif // _WIN32
 
-	/* Symbol may have version (Windows, FreeBSD), or not (Linux) */
+	/* Symbol may have version (Linux), or not (Windows, FreeBSD) */
 
 #define ICUUC_SYM(name)                                                   \
     do {                                                                  \
