@@ -623,6 +623,10 @@ TclOOMakeProcInstanceMethod(
     Interp *iPtr = (Interp *) interp;
     Proc *procPtr;
 
+    if (typePtr->version > TCL_OO_METHOD_VERSION_1) {
+	Tcl_Panic("%s: Wrong version in typePtr->version, should be %s",
+		"TclOOMakeProcInstanceMethod", "TCL_OO_METHOD_VERSION_1");
+    }
     if (TclCreateProc(interp, NULL, TclGetString(nameObj), argsObj, bodyObj,
 	    procPtrPtr) != TCL_OK) {
 	return NULL;
@@ -635,6 +639,45 @@ TclOOMakeProcInstanceMethod(
     return TclNewInstanceMethod(interp, (Tcl_Object) oPtr, nameObj, flags,
 	    typePtr, clientData);
 }
+
+Tcl_Method
+TclOOMakeProcInstanceMethod2(
+    Tcl_Interp *interp,		/* The interpreter containing the object. */
+    Object *oPtr,		/* The object to modify. */
+    int flags,			/* Whether this is a public method. */
+    Tcl_Obj *nameObj,		/* The name of the method, which _must not_ be
+				 * NULL. */
+    Tcl_Obj *argsObj,		/* The formal argument list for the method,
+				 * which _must not_ be NULL. */
+    Tcl_Obj *bodyObj,		/* The body of the method, which _must not_ be
+				 * NULL. */
+    const Tcl_MethodType2 *typePtr,
+				/* The type of the method to create. */
+    void *clientData,		/* The per-method type-specific data. */
+    Proc **procPtrPtr)		/* A pointer to the variable in which to write
+				 * the procedure record reference. Presumably
+				 * inside the structure indicated by the
+				 * pointer in clientData. */
+{
+    Interp *iPtr = (Interp *) interp;
+    Proc *procPtr;
+
+    if (typePtr->version < TCL_OO_METHOD_VERSION_2) {
+	Tcl_Panic("%s: Wrong version in typePtr->version, should be %s",
+		"TclOOMakeProcInstanceMethod2", "TCL_OO_METHOD_VERSION_2");
+    }
+    if (TclCreateProc(interp, NULL, TclGetString(nameObj), argsObj, bodyObj,
+	    procPtrPtr) != TCL_OK) {
+	return NULL;
+    }
+    procPtr = *procPtrPtr;
+    procPtr->cmdPtr = NULL;
+
+    InitCmdFrame(iPtr, procPtr);
+
+    return TclNewInstanceMethod(interp, (Tcl_Object) oPtr, nameObj, flags,
+	    (const Tcl_MethodType *)typePtr, clientData);
+}
 
 /*
  * ----------------------------------------------------------------------
@@ -675,6 +718,10 @@ TclOOMakeProcMethod(
     Interp *iPtr = (Interp *) interp;
     Proc *procPtr;
 
+    if (typePtr->version > TCL_OO_METHOD_VERSION_1) {
+	Tcl_Panic("%s: Wrong version in typePtr->version, should be %s",
+		"TclOOMakeProcMethod", "TCL_OO_METHOD_VERSION_1");
+    }
     if (TclCreateProc(interp, NULL, namePtr, argsObj, bodyObj,
 	    procPtrPtr) != TCL_OK) {
 	return NULL;
@@ -687,6 +734,49 @@ TclOOMakeProcMethod(
     return TclNewMethod(
 	    (Tcl_Class) clsPtr, nameObj, flags, typePtr, clientData);
 }
+
+Tcl_Method
+TclOOMakeProcMethod2(
+    Tcl_Interp *interp,		/* The interpreter containing the class. */
+    Class *clsPtr,		/* The class to modify. */
+    int flags,			/* Whether this is a public method. */
+    Tcl_Obj *nameObj,		/* The name of the method, which may be NULL;
+				 * if so, up to caller to manage storage
+				 * (e.g., because it is a constructor or
+				 * destructor). */
+    const char *namePtr,	/* The name of the method as a string, which
+				 * _must not_ be NULL. */
+    Tcl_Obj *argsObj,		/* The formal argument list for the method,
+				 * which _must not_ be NULL. */
+    Tcl_Obj *bodyObj,		/* The body of the method, which _must not_ be
+				 * NULL. */
+    const Tcl_MethodType2 *typePtr,
+				/* The type of the method to create. */
+    void *clientData,		/* The per-method type-specific data. */
+    Proc **procPtrPtr)		/* A pointer to the variable in which to write
+				 * the procedure record reference. Presumably
+				 * inside the structure indicated by the
+				 * pointer in clientData. */
+{
+    Interp *iPtr = (Interp *) interp;
+    Proc *procPtr;
+
+    if (typePtr->version < TCL_OO_METHOD_VERSION_2) {
+	Tcl_Panic("%s: Wrong version in typePtr->version, should be %s",
+		"TclOOMakeProcMethod2", "TCL_OO_METHOD_VERSION_2");
+    }
+    if (TclCreateProc(interp, NULL, namePtr, argsObj, bodyObj,
+	    procPtrPtr) != TCL_OK) {
+	return NULL;
+    }
+    procPtr = *procPtrPtr;
+    procPtr->cmdPtr = NULL;
+
+    InitCmdFrame(iPtr, procPtr);
+
+    return TclNewMethod(
+	    (Tcl_Class) clsPtr, nameObj, flags, (const Tcl_MethodType *)typePtr, clientData);
+}
 
 /*
  * ----------------------------------------------------------------------
@@ -1205,16 +1295,16 @@ RenderDeclarerName(
 
 #define LIMIT 60
 #define ELLIPSIFY(str,len) \
-	((len) > LIMIT ? LIMIT : (int)(len)), (str), ((len) > LIMIT ? "..." : "")
+    ((len) > LIMIT ? LIMIT : (int)(len)), (str), ((len) > LIMIT ? "..." : "")
 
 static void
 CommonMethErrorHandler(
     Tcl_Interp *interp,
     const char *special)
-
 {
     Tcl_Size objectNameLen;
-    CallContext *contextPtr = (CallContext *)((Interp *) interp)->varFramePtr->clientData;
+    CallContext *contextPtr = (CallContext *)((Interp *)
+	    interp)->varFramePtr->clientData;
     Method *mPtr = contextPtr->callPtr->chain[contextPtr->index].mPtr;
     const char *objectName, *kindName = "instance";
     Object *declarerPtr = NULL;
@@ -1222,17 +1312,14 @@ CommonMethErrorHandler(
     if (mPtr->declaringObjectPtr != NULL) {
 	declarerPtr = mPtr->declaringObjectPtr;
 	kindName = "object";
-
     } else if (mPtr->declaringClassPtr != NULL) {
-
-
 	declarerPtr = mPtr->declaringClassPtr->thisPtr;
 	kindName = "class";
     }
 
     if (declarerPtr) {
 	objectName = TclGetStringFromObj(TclOOObjectName(interp, declarerPtr),
-	    &objectNameLen);
+		&objectNameLen);
     } else {
 	objectName = "unknown or deleted";
 	objectNameLen = 18;
@@ -1241,13 +1328,14 @@ CommonMethErrorHandler(
 	Tcl_Size nameLen;
 	const char *methodName = TclGetStringFromObj(mPtr->namePtr, &nameLen);
 	Tcl_AppendObjToErrorInfo(interp, Tcl_ObjPrintf(
-	    "\n    (%s \"%.*s%s\" method \"%.*s%s\" line %d)",
-	    kindName, ELLIPSIFY(objectName, objectNameLen),
-	    ELLIPSIFY(methodName, nameLen), Tcl_GetErrorLine(interp)));
+		"\n    (%s \"%.*s%s\" method \"%.*s%s\" line %d)",
+		kindName, ELLIPSIFY(objectName, objectNameLen),
+		ELLIPSIFY(methodName, nameLen), Tcl_GetErrorLine(interp)));
     } else {
 	Tcl_AppendObjToErrorInfo(interp, Tcl_ObjPrintf(
-	    "\n    (%s \"%.*s%s\" %s line %d)", kindName,
-	    ELLIPSIFY(objectName, objectNameLen), special, Tcl_GetErrorLine(interp)));
+		"\n    (%s \"%.*s%s\" %s line %d)",
+		kindName, ELLIPSIFY(objectName, objectNameLen), special,
+		Tcl_GetErrorLine(interp)));
     }
 }
 
@@ -1255,8 +1343,8 @@ static void
 MethodErrorHandler(
     Tcl_Interp *interp,
     TCL_UNUSED(Tcl_Obj *) /*methodNameObj*/)
-	/* We pull the method name out of context instead of from argument */
 {
+    /* We pull the method name out of context instead of from argument. */
     CommonMethErrorHandler(interp, NULL);
 }
 
@@ -1264,62 +1352,20 @@ static void
 ConstructorErrorHandler(
     Tcl_Interp *interp,
     TCL_UNUSED(Tcl_Obj *) /*methodNameObj*/)
-	/* Ignore. We know it is the constructor. */
 {
-
-
-
-
-
+    /* We know this is for the constructor. */
     CommonMethErrorHandler(interp, "constructor");
-
-
-
-
-
-
 }
 
-
-
-
-
-
-
-
-
-
-
 static void
 DestructorErrorHandler(
     Tcl_Interp *interp,
     TCL_UNUSED(Tcl_Obj *) /*methodNameObj*/)
-	/* Ignore. We know it is the destructor. */
 {
-
-
-
-
-
+    /* We know this is for the destructor. */
     CommonMethErrorHandler(interp, "destructor");
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
 }
-
+
 /*
  * ----------------------------------------------------------------------
  *
@@ -1449,7 +1495,7 @@ TclOONewForwardInstanceMethod(
     if (prefixLen < 1) {
 	Tcl_SetObjResult(interp, Tcl_NewStringObj(
 		"method forward prefix must be non-empty", TCL_AUTO_LENGTH));
-	Tcl_SetErrorCode(interp, "TCL", "OO", "BAD_FORWARD", (char *)NULL);
+	OO_ERROR(interp, BAD_FORWARD);
 	return NULL;
     }
 
@@ -1488,7 +1534,7 @@ TclOONewForwardMethod(
     if (prefixLen < 1) {
 	Tcl_SetObjResult(interp, Tcl_NewStringObj(
 		"method forward prefix must be non-empty", TCL_AUTO_LENGTH));
-	Tcl_SetErrorCode(interp, "TCL", "OO", "BAD_FORWARD", (char *)NULL);
+	OO_ERROR(interp, BAD_FORWARD);
 	return NULL;
     }
 
