@@ -3,7 +3,7 @@
  *
  *	This file contains the bytecode optimizer.
  *
- * Copyright (c) 2013 by Donal Fellows.
+ * Copyright © 2013 Donal Fellows.
  *
  * See the file "license.terms" for information on usage and redistribution of
  * this file, and for a DISCLAIMER OF ALL WARRANTIES.
@@ -28,13 +28,13 @@ static void		TrimUnreachable(CompileEnv *envPtr);
  */
 
 #define DefineTargetAddress(tablePtr, address) \
-    ((void) Tcl_CreateHashEntry((tablePtr), (void *) (address), &isNew))
+    ((void) Tcl_CreateHashEntry((tablePtr), (address), &isNew))
 #define IsTargetAddress(tablePtr, address) \
     (Tcl_FindHashEntry((tablePtr), (void *) (address)) != NULL)
 #define AddrLength(address) \
     (tclInstructionTable[*(unsigned char *)(address)].numBytes)
 #define InstLength(instruction) \
-    (tclInstructionTable[(unsigned char)(instruction)].numBytes)
+    (tclInstructionTable[UCHAR(instruction)].numBytes)
 
 /*
  * ----------------------------------------------------------------------
@@ -54,7 +54,8 @@ LocateTargetAddresses(
     Tcl_HashTable *tablePtr)
 {
     unsigned char *currentInstPtr, *targetInstPtr;
-    int isNew, i;
+    int isNew;
+    Tcl_Size i;
     Tcl_HashEntry *hPtr;
     Tcl_HashSearch hSearch;
 
@@ -133,7 +134,7 @@ LocateTargetAddresses(
 	} else {
 	    targetInstPtr = envPtr->codeStart + rangePtr->breakOffset;
 	    DefineTargetAddress(tablePtr, targetInstPtr);
-	    if (rangePtr->continueOffset >= 0) {
+	    if (rangePtr->continueOffset != TCL_INDEX_NONE) {
 		targetInstPtr = envPtr->codeStart + rangePtr->continueOffset;
 		DefineTargetAddress(tablePtr, targetInstPtr);
 	    }
@@ -191,7 +192,7 @@ TrimUnreachable(
  * ConvertZeroEffectToNOP --
  *
  *	Replace PUSH/POP sequences (when non-hazardous) with NOPs. Also
- *	replace PUSH empty/CONCAT and TRY_CVT_NUMERIC (when followed by an
+ *	replace PUSH empty/STR_CONCAT and TRY_CVT_NUMERIC (when followed by an
  *	operation that guarantees the check for arithmeticity) and eliminate
  *	LNOT when we can invert the following JUMP condition.
  *
@@ -213,7 +214,7 @@ ConvertZeroEffectToNOP(
 
 	size = AddrLength(currentInstPtr);
 	while ((currentInstPtr + size < envPtr->codeNext)
-		&& *(currentInstPtr+size) == INST_NOP) {
+		&& currentInstPtr[size] == INST_NOP) {
 	    if (IsTargetAddress(&targets, currentInstPtr + size)) {
 		break;
 	    }
@@ -222,18 +223,18 @@ ConvertZeroEffectToNOP(
 	if (IsTargetAddress(&targets, currentInstPtr + size)) {
 	    continue;
 	}
-	nextInst = *(currentInstPtr + size);
+	nextInst = currentInstPtr[size];
 	switch (*currentInstPtr) {
 	case INST_PUSH1:
 	    if (nextInst == INST_POP) {
 		blank = size + InstLength(nextInst);
-	    } else if (nextInst == INST_CONCAT1
+	    } else if (nextInst == INST_STR_CONCAT1
 		    && TclGetUInt1AtPtr(currentInstPtr + size + 1) == 2) {
 		Tcl_Obj *litPtr = TclFetchLiteral(envPtr,
 			TclGetUInt1AtPtr(currentInstPtr + 1));
-		int numBytes;
+		Tcl_Size numBytes;
 
-		(void) Tcl_GetStringFromObj(litPtr, &numBytes);
+		(void) TclGetStringFromObj(litPtr, &numBytes);
 		if (numBytes == 0) {
 		    blank = size + InstLength(nextInst);
 		}
@@ -242,13 +243,13 @@ ConvertZeroEffectToNOP(
 	case INST_PUSH4:
 	    if (nextInst == INST_POP) {
 		blank = size + 1;
-	    } else if (nextInst == INST_CONCAT1
+	    } else if (nextInst == INST_STR_CONCAT1
 		    && TclGetUInt1AtPtr(currentInstPtr + size + 1) == 2) {
 		Tcl_Obj *litPtr = TclFetchLiteral(envPtr,
 			TclGetUInt4AtPtr(currentInstPtr + 1));
-		int numBytes;
+		Tcl_Size numBytes;
 
-		(void) Tcl_GetStringFromObj(litPtr, &numBytes);
+		(void) TclGetStringFromObj(litPtr, &numBytes);
 		if (numBytes == 0) {
 		    blank = size + InstLength(nextInst);
 		}
@@ -259,19 +260,19 @@ ConvertZeroEffectToNOP(
 	    switch (nextInst) {
 	    case INST_JUMP_TRUE1:
 		blank = size;
-		*(currentInstPtr + size) = INST_JUMP_FALSE1;
+		currentInstPtr[size] = INST_JUMP_FALSE1;
 		break;
 	    case INST_JUMP_FALSE1:
 		blank = size;
-		*(currentInstPtr + size) = INST_JUMP_TRUE1;
+		currentInstPtr[size] = INST_JUMP_TRUE1;
 		break;
 	    case INST_JUMP_TRUE4:
 		blank = size;
-		*(currentInstPtr + size) = INST_JUMP_FALSE4;
+		currentInstPtr[size] = INST_JUMP_FALSE4;
 		break;
 	    case INST_JUMP_FALSE4:
 		blank = size;
-		*(currentInstPtr + size) = INST_JUMP_TRUE4;
+		currentInstPtr[size] = INST_JUMP_TRUE4;
 		break;
 	    }
 	    break;
@@ -287,8 +288,6 @@ ConvertZeroEffectToNOP(
 	    case INST_INCR_ARRAY_STK:
 	    case INST_INCR_SCALAR_STK:
 	    case INST_INCR_STK:
-	    case INST_LOR:
-	    case INST_LAND:
 	    case INST_EQ:
 	    case INST_NEQ:
 	    case INST_LT:
@@ -319,7 +318,7 @@ ConvertZeroEffectToNOP(
 
 	if (blank > 0) {
 	    for (i=0 ; i<blank ; i++) {
-		*(currentInstPtr + i) = INST_NOP;
+		currentInstPtr[i] = INST_NOP;
 	    }
 	    size = blank;
 	}
@@ -344,23 +343,30 @@ AdvanceJumps(
     CompileEnv *envPtr)
 {
     unsigned char *currentInstPtr;
+    Tcl_HashTable jumps;
 
     for (currentInstPtr = envPtr->codeStart ;
 	    currentInstPtr < envPtr->codeNext-1 ;
 	    currentInstPtr += AddrLength(currentInstPtr)) {
-	int offset, delta;
+	int offset, delta, isNew;
 
 	switch (*currentInstPtr) {
 	case INST_JUMP1:
 	case INST_JUMP_TRUE1:
 	case INST_JUMP_FALSE1:
 	    offset = TclGetInt1AtPtr(currentInstPtr + 1);
+	    Tcl_InitHashTable(&jumps, TCL_ONE_WORD_KEYS);
 	    for (delta=0 ; offset+delta != 0 ;) {
 		if (offset + delta < -128 || offset + delta > 127) {
 		    break;
 		}
+		Tcl_CreateHashEntry(&jumps, INT2PTR(offset), &isNew);
+		if (!isNew) {
+		    offset = TclGetInt1AtPtr(currentInstPtr + 1);
+		    break;
+		}
 		offset += delta;
-		switch (*(currentInstPtr + offset)) {
+		switch (currentInstPtr[offset]) {
 		case INST_NOP:
 		    delta = InstLength(INST_NOP);
 		    continue;
@@ -373,14 +379,22 @@ AdvanceJumps(
 		}
 		break;
 	    }
+	    Tcl_DeleteHashTable(&jumps);
 	    TclStoreInt1AtPtr(offset, currentInstPtr + 1);
 	    continue;
 
 	case INST_JUMP4:
 	case INST_JUMP_TRUE4:
 	case INST_JUMP_FALSE4:
+	    Tcl_InitHashTable(&jumps, TCL_ONE_WORD_KEYS);
+	    Tcl_CreateHashEntry(&jumps, INT2PTR(0), &isNew);
 	    for (offset = TclGetInt4AtPtr(currentInstPtr + 1); offset!=0 ;) {
-		switch (*(currentInstPtr + offset)) {
+		Tcl_CreateHashEntry(&jumps, INT2PTR(offset), &isNew);
+		if (!isNew) {
+		    offset = TclGetInt4AtPtr(currentInstPtr + 1);
+		    break;
+		}
+		switch (currentInstPtr[offset]) {
 		case INST_NOP:
 		    offset += InstLength(INST_NOP);
 		    continue;
@@ -393,6 +407,7 @@ AdvanceJumps(
 		}
 		break;
 	    }
+	    Tcl_DeleteHashTable(&jumps);
 	    TclStoreInt4AtPtr(offset, currentInstPtr + 1);
 	    continue;
 	}
@@ -411,11 +426,11 @@ AdvanceJumps(
 
 void
 TclOptimizeBytecode(
-    CompileEnv *envPtr)
+    void *envPtr)
 {
-    ConvertZeroEffectToNOP(envPtr);
-    AdvanceJumps(envPtr);
-    TrimUnreachable(envPtr);
+    ConvertZeroEffectToNOP((CompileEnv *)envPtr);
+    AdvanceJumps((CompileEnv *)envPtr);
+    TrimUnreachable((CompileEnv *)envPtr);
 }
 
 /*
