@@ -207,6 +207,17 @@ int TclGetAliasObj(Tcl_Interp *interp, const char *childCmd,
 }
 #endif /* !defined(TCL_NO_DEPRECATED) */
 
+#define Tcl_CreateHashEntry createHashEntry
+static Tcl_HashEntry *
+Tcl_CreateHashEntry(
+    Tcl_HashTable *tablePtr,
+    const void *key,
+    int *newPtr)
+{
+    return (*((tablePtr)->createProc))(tablePtr, (const char *)(key), newPtr);
+}
+
+
 #define TclBN_mp_add mp_add
 #define TclBN_mp_add_d mp_add_d
 #define TclBN_mp_and mp_and
@@ -673,7 +684,7 @@ static const TclIntStubs tclIntStubs = {
     TclPtrObjMakeUpvar, /* 255 */
     TclPtrUnsetVar, /* 256 */
     TclStaticLibrary, /* 257 */
-    0, /* 258 */
+    TclMSB, /* 258 */
     0, /* 259 */
     0, /* 260 */
     TclUnusedStubEntry, /* 261 */
@@ -1241,7 +1252,7 @@ const TclStubs tclStubs = {
     0, /* 419 */
     0, /* 420 */
     0, /* 421 */
-    0, /* 422 */
+    Tcl_CreateHashEntry, /* 422 */
     Tcl_InitCustomHashTable, /* 423 */
     Tcl_InitObjHashTable, /* 424 */
     Tcl_CommandTraceInfo, /* 425 */
