@@ -3,16 +3,23 @@
  *
  *	Contains commands for platform specific tests on Windows.
  *
- * Copyright (c) 1996 Sun Microsystems, Inc.
+ * Copyright © 1996 Sun Microsystems, Inc.
  *
  * See the file "license.terms" for information on usage and redistribution of
  * this file, and for a DISCLAIMER OF ALL WARRANTIES.
  */
 
+#undef BUILD_tcl
+#undef STATIC_BUILD
 #ifndef USE_TCL_STUBS
 #   define USE_TCL_STUBS
 #endif
 #include "tclInt.h"
+#ifdef TCL_WITH_EXTERNAL_TOMMATH
+#   include "tommath.h"
+#else
+#   include "tclTomMath.h"
+#endif
 
 /*
  * For TestplatformChmod on Windows
@@ -94,7 +101,7 @@ TclplatformtestInit(
 
 static int
 TesteventloopCmd(
-    void *clientData,	/* Not used. */
+    TCL_UNUSED(void *),
     Tcl_Interp *interp,		/* Current interpreter. */
     int objc,			/* Number of arguments. */
     Tcl_Obj *const objv[])	/* Argument objects. */
@@ -102,10 +109,9 @@ TesteventloopCmd(
     static int *framePtr = NULL;/* Pointer to integer on stack frame of
 				 * innermost invocation of the "wait"
 				 * subcommand. */
-    (void)clientData;
 
-    if (objc < 2) {
-	Tcl_WrongNumArgs(interp, 1, objv, "option ...");
+    if (objc != 2) {
+	Tcl_WrongNumArgs(interp, 1, objv, "done|wait");
 	return TCL_ERROR;
     }
     if (strcmp(Tcl_GetString(objv[1]), "done") == 0) {
@@ -171,7 +177,7 @@ TesteventloopCmd(
 
 static int
 TestvolumetypeCmd(
-    void *clientData,	/* Not used. */
+    TCL_UNUSED(void *),
     Tcl_Interp *interp,		/* Current interpreter. */
     int objc,			/* Number of arguments. */
     Tcl_Obj *const objv[])	/* Argument objects. */
@@ -180,7 +186,6 @@ TestvolumetypeCmd(
     int found;
     char volType[VOL_BUF_SIZE];
     const char *path;
-    (void)clientData;
 
     if (objc > 2) {
 	Tcl_WrongNumArgs(interp, 1, objv, "?name?");
@@ -202,7 +207,7 @@ TestvolumetypeCmd(
     if (found == 0) {
 	Tcl_AppendResult(interp, "could not get volume type for \"",
 		(path?path:""), "\"", (char *)NULL);
-	TclWinConvertError(GetLastError());
+	Tcl_WinConvertError(GetLastError());
 	return TCL_ERROR;
     }
     Tcl_AppendResult(interp, volType, (char *)NULL);
@@ -238,7 +243,7 @@ TestvolumetypeCmd(
 
 static int
 TestwinclockCmd(
-    void *dummy,		/* Unused */
+    TCL_UNUSED(void *),
     Tcl_Interp* interp,		/* Tcl interpreter */
     int objc,			/* Argument count */
     Tcl_Obj *const objv[])	/* Argument vector */
@@ -251,7 +256,6 @@ TestwinclockCmd(
     Tcl_Obj *result;		/* Result of the command */
     LARGE_INTEGER t1, t2;
     LARGE_INTEGER p1, p2;
-    (void)dummy;
 
     if (objc != 1) {
 	Tcl_WrongNumArgs(interp, 1, objv, "");
@@ -272,11 +276,11 @@ TestwinclockCmd(
 
     result = Tcl_NewObj();
     Tcl_ListObjAppendElement(interp, result,
-	    Tcl_NewIntObj((int) (t2.QuadPart / 10000000)));
+	    Tcl_NewWideIntObj(t2.QuadPart / 10000000));
     Tcl_ListObjAppendElement(interp, result,
-	    Tcl_NewIntObj((int) ((t2.QuadPart / 10) % 1000000)));
-    Tcl_ListObjAppendElement(interp, result, Tcl_NewIntObj(tclTime.sec));
-    Tcl_ListObjAppendElement(interp, result, Tcl_NewIntObj(tclTime.usec));
+	    Tcl_NewWideIntObj((t2.QuadPart / 10) % 1000000));
+    Tcl_ListObjAppendElement(interp, result, Tcl_NewWideIntObj(tclTime.sec));
+    Tcl_ListObjAppendElement(interp, result, Tcl_NewWideIntObj(tclTime.usec));
 
     Tcl_ListObjAppendElement(interp, result, Tcl_NewWideIntObj(p1.QuadPart));
     Tcl_ListObjAppendElement(interp, result, Tcl_NewWideIntObj(p2.QuadPart));
@@ -288,13 +292,12 @@ TestwinclockCmd(
 
 static int
 TestwinsleepCmd(
-    void *clientData,	/* Unused */
+    TCL_UNUSED(void *),
     Tcl_Interp* interp,		/* Tcl interpreter */
     int objc,			/* Parameter count */
     Tcl_Obj *const * objv)	/* Parameter vector */
 {
     int ms;
-    (void)clientData;
 
     if (objc != 2) {
 	Tcl_WrongNumArgs(interp, 1, objv, "ms");
@@ -332,10 +335,10 @@ TestwinsleepCmd(
 
 static int
 TestExceptionCmd(
-    void *dummy,			/* Unused */
-    Tcl_Interp* interp,			/* Tcl interpreter */
-    int objc,				/* Argument count */
-    Tcl_Obj *const objv[])		/* Argument vector */
+    TCL_UNUSED(void *),
+    Tcl_Interp* interp,		/* Tcl interpreter */
+    int objc,			/* Argument count */
+    Tcl_Obj *const objv[])	/* Argument vector */
 {
     static const char *const cmds[] = {
 	"access_violation", "datatype_misalignment", "array_bounds",
@@ -359,7 +362,6 @@ TestExceptionCmd(
 	EXCEPTION_GUARD_PAGE, EXCEPTION_INVALID_HANDLE, CONTROL_C_EXIT
     };
     int cmd;
-    (void)dummy;
 
     if (objc != 2) {
 	Tcl_WrongNumArgs(interp, 0, objv, "<type-of-exception>");
@@ -442,9 +444,9 @@ TestplatformChmod(
     res = -1; /* Assume failure */
 
     Tcl_DStringInit(&ds);
-    Tcl_UtfToExternalDString(NULL, nativePath, -1, &ds);
+    Tcl_UtfToChar16DString(nativePath, -1, &ds);
 
-    attr = GetFileAttributesA(Tcl_DStringValue(&ds));
+    attr = GetFileAttributesW((WCHAR *)Tcl_DStringValue(&ds));
     if (attr == 0xFFFFFFFF) {
 	goto done; /* Not found */
     }
@@ -460,15 +462,15 @@ TestplatformChmod(
 	    && GetLastError() != ERROR_INSUFFICIENT_BUFFER) {
 	goto done;
     }
-    pTokenUser = (TOKEN_USER *)ckalloc(dw);
+    pTokenUser = (TOKEN_USER *)Tcl_Alloc(dw);
     if (!GetTokenInformation(hToken, TokenUser, pTokenUser, dw, &dw)) {
 	goto done;
     }
     aceEntry[nSids].sidLen = GetLengthSid(pTokenUser->User.Sid);
-    aceEntry[nSids].pSid = (PSID)ckalloc(aceEntry[nSids].sidLen);
+    aceEntry[nSids].pSid = (PSID)Tcl_Alloc(aceEntry[nSids].sidLen);
     if (!CopySid(aceEntry[nSids].sidLen, aceEntry[nSids].pSid,
 	    pTokenUser->User.Sid)) {
-	ckfree(aceEntry[nSids].pSid); /* Since we have not ++'ed nSids */
+	Tcl_Free(aceEntry[nSids].pSid); /* Since we have not ++'ed nSids */
 	goto done;
     }
     /*
@@ -501,19 +503,19 @@ TestplatformChmod(
 		GetLastError() != ERROR_INSUFFICIENT_BUFFER) {
 	    goto done;
 	}
-	pTokenGroup = (TOKEN_PRIMARY_GROUP *)ckalloc(dw);
+	pTokenGroup = (TOKEN_PRIMARY_GROUP *)Tcl_Alloc(dw);
 	if (!GetTokenInformation(hToken, TokenPrimaryGroup, pTokenGroup, dw, &dw)) {
-	    ckfree(pTokenGroup);
+	    Tcl_Free(pTokenGroup);
 	    goto done;
 	}
 	aceEntry[nSids].sidLen = GetLengthSid(pTokenGroup->PrimaryGroup);
-	aceEntry[nSids].pSid = (PSID)ckalloc(aceEntry[nSids].sidLen);
+	aceEntry[nSids].pSid = (PSID)Tcl_Alloc(aceEntry[nSids].sidLen);
 	if (!CopySid(aceEntry[nSids].sidLen, aceEntry[nSids].pSid, pTokenGroup->PrimaryGroup)) {
-	    ckfree(pTokenGroup);
-	    ckfree(aceEntry[nSids].pSid); /* Since we have not ++'ed nSids */
+	    Tcl_Free(pTokenGroup);
+	    Tcl_Free(aceEntry[nSids].pSid); /* Since we have not ++'ed nSids */
 	    goto done;
 	}
-	ckfree(pTokenGroup);
+	Tcl_Free(pTokenGroup);
 
 	/* Generate mask for group ACL */
 
@@ -537,10 +539,10 @@ TestplatformChmod(
 	    goto done;
 	}
 	aceEntry[nSids].sidLen = GetLengthSid(pWorldSid);
-	aceEntry[nSids].pSid = (PSID)ckalloc(aceEntry[nSids].sidLen);
+	aceEntry[nSids].pSid = (PSID)Tcl_Alloc(aceEntry[nSids].sidLen);
 	if (!CopySid(aceEntry[nSids].sidLen, aceEntry[nSids].pSid, pWorldSid)) {
 	    LocalFree(pWorldSid);
-	    ckfree(aceEntry[nSids].pSid); /* Since we have not ++'ed nSids */
+	    Tcl_Free(aceEntry[nSids].pSid); /* Since we have not ++'ed nSids */
 	    goto done;
 	}
 	LocalFree(pWorldSid);
@@ -566,9 +568,9 @@ TestplatformChmod(
     /* Add in size required for each ACE entry in the ACL */
     for (i = 0; i < nSids; ++i) {
 	newAclSize +=
-	    TclOffset(ACCESS_ALLOWED_ACE, SidStart) + aceEntry[i].sidLen;
+	    offsetof(ACCESS_ALLOWED_ACE, SidStart) + aceEntry[i].sidLen;
     }
-    newAcl = (PACL)ckalloc(newAclSize);
+    newAcl = (PACL)Tcl_Alloc(newAclSize);
     if (!InitializeAcl(newAcl, newAclSize, ACL_REVISION)) {
 	goto done;
     }
@@ -584,7 +586,7 @@ TestplatformChmod(
      * to remove inherited ACL (we need to overwrite the default ACL's in this case)
      */
 
-    if (SetNamedSecurityInfoA((LPSTR)Tcl_DStringValue(&ds), SE_FILE_OBJECT,
+    if (SetNamedSecurityInfoW((LPWSTR)Tcl_DStringValue(&ds), SE_FILE_OBJECT,
 	    DACL_SECURITY_INFORMATION | PROTECTED_DACL_SECURITY_INFORMATION,
 	    NULL, NULL, newAcl, NULL) == ERROR_SUCCESS) {
 	res = 0;
@@ -592,21 +594,21 @@ TestplatformChmod(
 
   done:
     if (pTokenUser) {
-	ckfree(pTokenUser);
+	Tcl_Free(pTokenUser);
     }
     if (hToken) {
 	CloseHandle(hToken);
     }
     if (newAcl) {
-	ckfree(newAcl);
+	Tcl_Free(newAcl);
     }
     for (i = 0; i < nSids; ++i) {
-	ckfree(aceEntry[i].pSid);
+	Tcl_Free(aceEntry[i].pSid);
     }
 
     if (res == 0) {
 	/* Run normal chmod command */
-	res = _chmod(Tcl_DStringValue(&ds), pmode);
+	res = _wchmod((WCHAR*)Tcl_DStringValue(&ds), pmode);
     }
     Tcl_DStringFree(&ds);
     return res;
@@ -633,13 +635,12 @@ TestplatformChmod(
 
 static int
 TestchmodCmd(
-    void *dummy,		/* Not used. */
+    TCL_UNUSED(void *),
     Tcl_Interp *interp,		/* Current interpreter. */
     int objc,			/* Parameter count */
     Tcl_Obj *const * objv)	/* Parameter vector */
 {
     int i, mode;
-    (void)dummy;
 
     if (objc < 2) {
 	Tcl_WrongNumArgs(interp, 1, objv, "mode file ?file ...?");
